@@ -1,8 +1,6 @@
 use std::ops::{Add, Sub, Mul, Neg};
 
-use crate::point::Point;
-
-use super::dot;
+use crate::point::{Vec2D, Unit};
 
 /// Stores the angle in radians internally
 #[derive(Copy, Clone, Debug, Default, PartialEq)]
@@ -29,17 +27,17 @@ impl Angle {
         Angle(degrees.to_radians())
     }
 
-    pub fn between<P: Point>(a: P, b: P) -> Angle {
+    pub fn between<T: Unit>(a: Vec2D<T>, b: Vec2D<T>) -> Angle {
         if a == b {
             return Angle::from_radians(0.0);
         }
 
-        let angle = (dot(a, b)/(a.magnitude() * b.magnitude())).acos();
+        let angle = (a.dot(b)/(a.magnitude() * b.magnitude())).acos();
 
-        Self::from_radians(if a.x() * b.y() - a.y() * b.x() < 0.0 {
-            -angle
+        Self::from_radians(if a.x.val() * b.y.val() - a.y.val() * b.x.val() < 0.0 {
+            (-angle).val()
         } else {
-            angle
+            angle.val()
         })
     }
 }
